#!/usr/bin/perl -w

use Errno qw(EAGAIN);

#run the bgpd.pl and make it send ann-beacon on the hour and wd-beacon 
#on the half hour. 

if (($#ARGV>=0) && ($ARGV[0] eq "-h")) {
    print "Usage wrapper.pl [optional: <bgpd.pl config file name>] [optional: <bgpd.pl log file name>]\n";
    exit(1);
}

$configFile = ($#ARGV>=0) ? $ARGV[0] : "bgpd.conf";
$logFile = ($#ARGV>=1) ? $ARGV[1] : "bgpd.log";

$period=30; #30 minutes;
$pidFile="/var/run/bgpd_pl.pid";


# fork the bgpd.pl process: bgpd.pl --log [log file name] --daemon --config [config file name]
# get process ID of the bgpd.pl, send USR1/USR2 signal on the hour and on the half hour. 

FORK: {
    if ($pid = fork) {
	#parent here
	print localtime().":waiting 30 seconds for bgpd.pl to start up. \n";

	sleep(30); #wait until connection is established.

	open(FPID, $pidFile) || die "Can't open pid file:$pidFile: $!";
	$line=<FPID>; $line=~/^(\d+)/;
	$pid=$1;
	print localtime()."bgpd.pl's pid is $pid.\n";

	#send the first update:
	print  localtime().":ANN: kill -s USR1 $pid.\n";
	system("kill -s USR1 $pid");

	$doAnn=1;

	while(1){
	    ($min, $sec) = (localtime) [1,0];
	    #print " time: min:$min, sec:$sec.\n"; 
	    if( (int $min/$period)%2==0) {$doAnn=0;}
	    else {$doAnn=1;}

	    $sleepTime = 60*($period*((int $min/$period)+1) -$min) - $sec;
	    $sleepTime = ($sleepTime<0) ? 0 : $sleepTime;
	    print  localtime().":sleep for $sleepTime seconds: $doAnn.\n";
	    sleep($sleepTime);
	    
	    if($doAnn==1) { 
		print  localtime().":ANN: kill -s USR1 $pid\n";
		system("kill -s USR1 $pid"); $doAnn=0;
	    }
	    else { 
		print  localtime().":WD: kill -s USR2 $pid\n";
		system("kill -s USR2 $pid"); $doAnn=1;
	    }

	}	
    } elsif (defined $pid) {
	#child here  
	print  localtime().":exec bgpd.pl: $$.\n";
	exec "perl -w bgpd.pl --log $logFile --daemon --config $configFile" or die "Can't exec: $!";
	exit(1);
    } elsif ($! == EAGAIN) {
	print "sleeping...before reforking.\n";
	sleep 5;
	redo FORK;
    } else {
	#weird fork error
	die "Can't fork: $!\n";
    }
}

